@@ -376,6 +376,9 @@ static void setup_child_environ(struct uftrace_opts *opts, int argc, char *argv[
 	put_libmcount_path(libpath);
 	setenv("XRAY_OPTIONS", "patch_premain=false", 1);
 	setenv("GLIBC_TUNABLES", "glibc.cpu.hwcaps=-IBT,-SHSTK", 1);
+
+	/* disable debuginfo daemon */
+	unsetenv("DEBUGINFOD_URLS");
 }
 
 static uint64_t calc_feat_mask(struct uftrace_opts *opts)
@@ -2005,7 +2008,8 @@ static void copy_data_files(struct uftrace_opts *opts, const char *ext)
 	glob(path, GLOB_NOSORT, NULL, &g);
 
 	for (i = 0; i < g.gl_pathc; i++) {
-		snprintf(path, sizeof(path), "%s/%s", opts->dirname, basename(g.gl_pathv[i]));
+		snprintf(path, sizeof(path), "%s/%s", opts->dirname,
+			 uftrace_basename(g.gl_pathv[i]));
 		copy_file(g.gl_pathv[i], path);
 	}
 
