@@ -745,7 +745,7 @@ static void mcount_init_file(void)
 
 	send_session_msg(&mtd, mcount_session_name());
 	pr_dbg("new session started: %.*s: %s\n", SESSION_ID_LEN, mcount_session_name(),
-	       basename(mcount_exename));
+	       uftrace_basename(mcount_exename));
 
 	sigemptyset(&sa.sa_mask);
 	sigaction(SIGABRT, &sa, &old_sigact[0]);
