@@ -181,6 +181,17 @@ static void release_debug_file(struct rb_root *root)
 #include <gelf.h>
 #include <libelf.h>
 
+/*
+ * Used by libdwfl in build_debug_info_cb after setup_dwarf_info returns,
+ * so this may not be allocated temporary on setup_debug_info's stack (which is freed)
+ */
+static char *debuginfo_path = NULL;
+Dwfl_Callbacks dwfl_callbacks = {
+	.find_elf = dwfl_linux_proc_find_elf,
+	.find_debuginfo = dwfl_standard_find_debuginfo,
+	.debuginfo_path = &debuginfo_path,
+};
+
 /*
  * symbol table contains normalized (zero-based) relative address.
  * but some other info in non-PIE executable has different base
@@ -271,12 +282,12 @@ static int setup_dwarf_info(const char *filename, struct uftrace_dbg_info *dinfo
 			    unsigned long offset, bool force)
 {
 	int fd;
+	Dwfl_Module *mod;
+	Dwarf_Addr bias;
 
 	if (force || check_trace_functions(filename) != TRACE_CYGPROF)
 		dinfo->needs_args = true;
 
-	pr_dbg2("setup dwarf debug info for %s\n", filename);
-
 	fd = open(filename, O_RDONLY);
 	if (fd < 0) {
 		pr_dbg2("cannot open debug info for %s: %m\n", filename);
@@ -284,13 +295,26 @@ static int setup_dwarf_info(const char *filename, struct uftrace_dbg_info *dinfo
 	}
 
 	dinfo->dw = dwarf_begin(fd, DWARF_C_READ);
-	close(fd);
+	if (dinfo->dw != NULL)
+		goto ok;
 
-	if (dinfo->dw == NULL) {
-		pr_dbg2("failed to setup debug info: %s\n", dwarf_errmsg(dwarf_errno()));
+	dinfo->dwfl = dwfl_begin(&dwfl_callbacks);
+	if (dinfo->dwfl == NULL) {
+		pr_dbg2("failed to begin libdwfl: %s\n", dwfl_errmsg(dwfl_errno()));
+		close(fd);
 		return -1;
 	}
 
+	mod = dwfl_report_offline(dinfo->dwfl, filename, filename, fd);
+	dinfo->dw = dwfl_module_getdwarf(mod, &bias);
+	if (dinfo->dw == NULL) {
+		pr_dbg2("failed to setup debug info: %s\n", dwfl_errmsg(dwfl_errno()));
+		dwfl_end(dinfo->dwfl);
+		dinfo->dwfl = NULL;
+		return -1;
+	}
+
+ok:
 	pr_dbg2("setup dwarf debug info for %s\n", filename);
 
 	/*
@@ -308,11 +332,17 @@ static int setup_dwarf_info(const char *filename, struct uftrace_dbg_info *dinfo
 
 static void release_dwarf_info(struct uftrace_dbg_info *dinfo)
 {
-	if (dinfo->dw == NULL)
-		return;
+	if (dinfo->dwfl != NULL) {
+		/* this will also free dinfo->dw */
+		dwfl_end(dinfo->dwfl);
+		dinfo->dwfl = NULL;
+		dinfo->dw = NULL;
+	}
 
-	dwarf_end(dinfo->dw);
-	dinfo->dw = NULL;
+	if (dinfo->dw != NULL) {
+		dwarf_end(dinfo->dw);
+		dinfo->dw = NULL;
+	}
 }
 
 #define ARGSPEC_MAX_SIZE 256
@@ -489,7 +519,7 @@ static char *make_enum_name(Dwarf_Die *die)
 
 	off = dwarf_cuoffset(die);
 
-	xasprintf(&enum_name, "_%s_%lx", basename(cu_name), off);
+	xasprintf(&enum_name, "_%s_%lx", uftrace_basename(cu_name), off);
 
 	/* replace forbidden characters */
 	tmp = enum_name;
@@ -504,7 +534,6 @@ static size_t type_size(Dwarf_Die *die, size_t default_size)
 {
 	Dwarf_Word size;
 
-	/* require >= elfutils 0.144 */
 	if (dwarf_aggregate_size(die, &size) >= 0)
 		return size * 8;
 
@@ -876,6 +905,7 @@ static bool resolve_type_info(Dwarf_Die *die, struct arg_data *ad, struct type_d
 		case DW_FORM_ref_udata:
 		case DW_FORM_ref_addr:
 		case DW_FORM_ref_sig8:
+		case DW_FORM_GNU_ref_alt:
 			dwarf_formref_die(&type, &ref);
 			die = &ref;
 			break;
@@ -886,6 +916,9 @@ static bool resolve_type_info(Dwarf_Die *die, struct arg_data *ad, struct type_d
 
 		switch (dwarf_tag(die)) {
 		case DW_TAG_enumeration_type:
+			if (td->pointer)
+				break;
+
 			enum_str = fill_enum_str(die);
 			if (enum_str == NULL)
 				return false; /* use default format */
@@ -1690,7 +1723,8 @@ static void build_dwarf_info(struct uftrace_dbg_info *dinfo, struct uftrace_symt
 		if (dwarf_offdie(dinfo->dw, curr + header_sz, &cudie) == NULL)
 			break;
 
-		if (dwarf_tag(&cudie) != DW_TAG_compile_unit)
+		if (dwarf_tag(&cudie) != DW_TAG_compile_unit &&
+		    dwarf_tag(&cudie) != DW_TAG_partial_unit)
 			break;
 
 		if (uftrace_done)
@@ -1739,6 +1773,7 @@ static int setup_dwarf_info(const char *filename, struct uftrace_dbg_info *dinfo
 			    unsigned long offset, bool force)
 {
 	dinfo->dw = NULL;
+	dinfo->dwfl = NULL;
 	return 0;
 }
 
@@ -1753,16 +1788,10 @@ static void release_dwarf_info(struct uftrace_dbg_info *dinfo)
 
 #endif /* !HAVE_LIBDW */
 
-int setup_debug_info(const char *filename, struct uftrace_dbg_info *dinfo, unsigned long offset,
-		     bool force)
+static void setup_debug_info(struct uftrace_dbg_info *dinfo)
 {
-	dinfo->args = RB_ROOT;
-	dinfo->rets = RB_ROOT;
-	dinfo->enums = RB_ROOT;
-	dinfo->files = RB_ROOT;
+	memset(dinfo, 0, sizeof(*dinfo));
 	dinfo->loaded = true;
-
-	return setup_dwarf_info(filename, dinfo, offset, force);
 }
 
 static void release_debug_info(struct uftrace_dbg_info *dinfo)
@@ -1776,8 +1805,8 @@ static void release_debug_info(struct uftrace_dbg_info *dinfo)
 	dinfo->locs = NULL;
 
 	free(dinfo->base_dir);
+	dinfo->base_dir = NULL;
 
-	release_dwarf_info(dinfo);
 	dinfo->loaded = false;
 }
 
@@ -1850,8 +1879,11 @@ void prepare_debug_info(struct uftrace_sym_info *sinfo, enum uftrace_pattern_typ
 		if (map->mod == NULL || map->mod->dinfo.loaded)
 			continue;
 
-		setup_debug_info(map->libname, dinfo, map->start, force);
+		setup_debug_info(dinfo);
+
+		setup_dwarf_info(map->libname, dinfo, map->start, force);
 		build_dwarf_info(dinfo, stab, ptype, &dwarf_args, &dwarf_rets);
+		release_dwarf_info(dinfo);
 	}
 
 	strv_free(&dwarf_args);
@@ -1903,7 +1935,7 @@ static FILE *create_debug_file(const char *dirname, const char *filename, char *
 	FILE *fp;
 	char *tmp;
 
-	xasprintf(&tmp, "%s/%s.dbg", dirname, basename(filename));
+	xasprintf(&tmp, "%s/%s.dbg", dirname, uftrace_basename(filename));
 	if (match_debug_file(tmp, filename, build_id)) {
 		free(tmp);
 		return NULL;
@@ -2046,7 +2078,7 @@ static void save_debug_entries(struct uftrace_dbg_info *dinfo, const char *dirna
 			save_debug_file(fp, 'R', entry->spec, 0);
 	}
 
-	close_debug_file(fp, dirname, basename(filename), build_id);
+	close_debug_file(fp, dirname, uftrace_basename(filename), build_id);
 }
 
 void save_debug_info(struct uftrace_sym_info *sinfo, const char *dirname)
@@ -2073,7 +2105,7 @@ static int load_debug_file(struct uftrace_dbg_info *dinfo, struct uftrace_symtab
 	char *func = NULL;
 	uint64_t offset = 0;
 
-	xasprintf(&pathname, "%s/%s.dbg", dirname, basename(filename));
+	xasprintf(&pathname, "%s/%s.dbg", dirname, uftrace_basename(filename));
 
 	if (!match_debug_file(pathname, filename, build_id)) {
 		char *newfile;
