@@ -443,9 +443,16 @@ static void skip_kernel_functions(struct uftrace_kernel_writer *kernel)
 		 * messes up the trace output so it'd be better hiding them.
 		 */
 		"smp_irq_work_interrupt",
-		/* Disable syscall tracing in the kernel */
-		"syscall_trace_enter_phase1",
-		"syscall_slow_exit_work",
+		/*
+		 * Disable syscall tracing in the kernel.  Note that the
+		 * kernel uses glob patterns internally, so it can use
+		 * the following regardless of the uftrace --match value.
+		 */
+		"syscall_*",
+		/*
+		 * This function is called whenever it returns to userspace
+		 * in order to process things like signal handling.
+		 */
 		"exit_to_user_mode_prepare",
 #ifdef __aarch64__
 		/*
@@ -752,7 +759,7 @@ static int save_event_files(struct uftrace_kernel_writer *kernel, FILE *fp)
 	DIR *event = NULL;
 	struct dirent *sys, *name;
 
-	if (read_tracing_file("events/enable", buf, sizeof(buf)))
+	if (read_tracing_file("events/enable", buf, sizeof(buf)) < 0)
 		goto out;
 
 	/* no events enabled: exit */
@@ -786,9 +793,13 @@ static int save_event_files(struct uftrace_kernel_writer *kernel, FILE *fp)
 		if (buf[0] == '0')
 			continue;
 
+		put_tracing_file(filename);
 		snprintf(buf, sizeof(buf), "events/%s", sys->d_name);
+		filename = get_tracing_file(buf);
+		if (filename == NULL)
+			goto out;
 
-		event = opendir(buf);
+		event = opendir(filename);
 		if (event == NULL)
 			goto out;
 
